#!/usr/bin/env python
"""
Usage:
    package.py projectname major minor [destination1 [destination2...]]
Example:
    package.py wxbanker 0.6 .0.0~ppa1 ppa1 ppa2 lp
"""
APP = []
import commands, os
from config import PKGROOT, CODEROOT, PPAS, DISTS

def replace(command):
    replacements = (("ROOT", CODEROOT), ("APP", APP[0]), ("PKGROOT", PKGROOT))
    for variable, replacement in replacements:
        command = command.replace("[%s]"%variable, replacement)
    return command

def perform(cmds):
    for command in cmds:
        command = replace(command)
        print command
        status, output = commands.getstatusoutput(command)
        if output:
            print output
        assert status == 0, (command, output)

def prepare(version):
    commands = [
        "mkdir -p [PKGROOT]",
        "rm -rf [PKGROOT]/*",
        "bzr export [PKGROOT]/[APP] [ROOT]/[APP]-%s" % version
        ]
    perform(commands)

    # If the package doesn't contain a debian already, assume it is a separate thing to export.
    pdir = os.path.expanduser(replace("[PKGROOT]/[APP]/debian"))
    if not os.path.exists(pdir):
        perform(["bzr export [PKGROOT]/[APP]/debian [ROOT]/packaging-%s" % version])


def dch(version, dist=""):
    if dist:
        distMod = "~%s" % dist
    else:
        distMod = ""
        dist = sorted(DISTS.keys())[-1]
        print "Latest:", dist

    perform(["dch -b -v %s%s -c [PKGROOT]/[APP]/debian/changelog -D %s \"automated upload\"" % (version, distMod, DISTS[dist])])

def watermark(mark):
    for script in ("postinst", "postrm"):
        inst = replace(os.path.expanduser("[PKGROOT]/[APP]/debian/%s"%script))
        # If there isn't a postinst or preinst, we can't watermark of course.
        if not os.path.exists(inst):
            return False

        lines = open(inst).readlines()
        assert lines[2].startswith("VER=")
        lines[2] = "VER=%s\n"%mark
        open(inst, "w").writelines(lines)
        print "Watermarked as as: %s" % mark

def make_unsigned_deb():
    perform(["cd [PKGROOT]/[APP] && debuild -i -us -uc -b"])

def make_source():
    perform(["cd [PKGROOT]/[APP] && debuild -S -sa"])

def make_tgz(major, version):
    name = "[APP]-%s"%version
    perform([
        "bzr export [PKGROOT]/[APP]-%s [ROOT]/[APP]-%s" % (version, major),
        "cd [PKGROOT] && tar -cz %s --file %s.tar.gz" % (name, name),
        ])

def gen_sig(filepath):
    dir, file = os.path.split(filepath)
    perform(["cd %s && gpg --armor --sign --detach-sig %s" % (dir, file)])

def upload(ppa, dist=""):
    perform(["dput --force ppa:%s [PKGROOT]/*%s_source.changes" % (ppa, dist)])
    print "Uploaded to %s" % ppa

def share():
    perform([
        "rm -rf ~/Public/packaging",
        "cp -r [PKGROOT] ~/Public/",
        ])


def main(major, minor, destinations):
    version = major+minor
    prepare(major)
    # Iterate over each destination (PPA) to upload to.
    # Use "lp" destination (default) to build a .deb and .tar.gz
    if destinations != ["lp"] and destinations != ["test"]: # Only warn if this is actually going to PPA[s]
        raw_input("Will create %s%s and upload to %s for %s. Enter to continue..." % (major, minor, destinations, DISTS.keys()))
    for dest in destinations:
        watermarked = watermark("%s%s~%s"%(major,minor,dest))
        if dest in ["lp", "test"]:
            dch(version)
            make_unsigned_deb()
            make_tgz(major, version)
            # We only need to sign 'lp' releases.
            if dest == "lp":
                gen_sig("[PKGROOT]/[APP]_%s_all.deb"%version)
                gen_sig("[PKGROOT]/[APP]-%s.tar.gz"%version)
        else:
            ppa = PPAS[dest]
            # Upload for each distribution supported.
            for dist in DISTS:
                dch(version, dist)
                make_source()
                upload(ppa, dist)
        print "Watermarked: %s" % bool(watermarked)
    share()

if __name__ == "__main__":
    import sys
    app, major, minor = sys.argv[1:4]
    APP.append(app)

    destinations = ["test"]
    if len(sys.argv) > 4:
        destinations = sys.argv[4:]

    main(major, minor, destinations)
