import launchpadlib, datetime, os
from launchpadlib.launchpad import Launchpad

cachedir = os.path.expanduser("~/.launchpadlib/cache")
server = launchpadlib.launchpad.STAGING_SERVICE_ROOT

class Releaser:
    def __init__(self, lp, projectName, version):
        self.LP = lp
        self.ProjectName = projectName
        self.Version = version
        self._Initialize()

    def _Initialize(self):
        # Grab the project object.
        self.Project = self.LP.projects[projectName]
        # Grab the specified milestone.
        self.Milestone = self.Project.getMilestone(name=version)
        self.ReleaseNotes = self.Milestone.summary
        self.ReleaseDate = datetime.date.today().strftime('%Y-%m-%d')

    def Perform(self, assertReady=True):
        if assertReady:
            self.AssertReady()
        numFixed = self.MarkCommittedAsReleased()
        print "Marked %s bugs Fix Committed -> Fix Released." % numFixed
        self.MakeReleaseFromMilestone()

    def AssertReady(self):
        """Raise an exception if there are any open bugs for this milestone."""
        unfixed = [task for task in self.Milestone.searchTasks() if task.status not in ["Fix Committed", "Fix Released"]]
        if unfixed:
            raise Exception("There are still %s unfixed tasks for %s, cannot release! Please mark them as Fix Committed or move them to another Milestone." % (len(unfixed), self.Milestone.name))

    def MarkCommittedAsReleased(self):
        """Mark "Fix Committed" bugs as "Fix Released"."""
        committedBugs = [task for task in self.Milestone.searchTasks(status="Fix Committed")]
        for committed in committedBugs:
            committed.transitionToStatus(status="Fix Released")
        return len(committedBugs)

    def MakeReleaseFromMilestone(self):
        release = self.Milestone.createProductRelease(date_released=self.ReleaseDate, release_notes=self.ReleaseNotes)


def main(server, lpuser, projectName, version):
    # Log in.
    lp = Launchpad.login_with(lpuser, server, cachedir)
    releaser = Releaser(lp, projectName, version)
    releaser.Perform(assertReady=False)

if __name__ == "__main__":
    lpuser = "mrooney"
    projectName = "wxbanker"
    version = "0.7"
    main(server, lpuser, projectName, version)
